@@ -16,13 +16,14 @@ import unittest
 
 import numpy as np
 
-from obspy import Trace, UTCDateTime, read
+from obspy import Trace, UTCDateTime, read, read_inventory
 from obspy.core.util.base import NamedTemporaryFile
-from obspy.core.util.misc import CatchOutput
+from obspy.core.util.misc import SuppressOutput
 from obspy.io.sac import attach_paz
 from obspy.signal.headers import clibevresp
-from obspy.signal.invsim import (cosine_taper, estimate_magnitude, evalresp,
-                                 simulate_seismometer)
+from obspy.signal.invsim import (
+    cosine_taper, estimate_magnitude, evalresp, simulate_seismometer,
+    evalresp_for_frequencies)
 
 
 # Seismometers defined as in Pitsa with one zero less. The corrected
@@ -150,6 +151,10 @@ class InvSimTestCase(unittest.TestCase):
             RTBE PITSA 1.325 ObsPy 1.363
             RMOA PITSA 1.629 ObsPy 1.675
         """
+        # the poles/zeros are the same for all three stations but the overall
+        # sensitivity differs, this was probably not taken into account when
+        # implementing this test (the specified 'sensitivity' is for RTSH), so
+        # below we use the response for station RTSH for the test
         paz = {'poles': [-4.444 + 4.444j, -4.444 - 4.444j, -1.083 + 0j],
                'zeros': [0 + 0j, 0 + 0j, 0 + 0j],
                'gain': 1.0,
@@ -161,6 +166,20 @@ class InvSimTestCase(unittest.TestCase):
         mag_rnon = estimate_magnitude(paz, 6.78e4, 0.125, 1.538)
         self.assertAlmostEqual(mag_rnon, 1.4995311686507182)
 
+        # now also test using Response object to calculate amplitude
+        # (use RTSH response for all three measurements, see above comment)
+        # response calculated using all stages is slightly different from the
+        # PAZ + overall sensitivity used above, so we get slightly different
+        # values here..
+        response = read_inventory(os.path.join(self.path, 'BW_RTSH.xml'),
+                                  format='STATIONXML')[0][0][0].response
+        mag_rtsh = estimate_magnitude(response, 3.34e6, 0.065, 0.255)
+        self.assertAlmostEqual(mag_rtsh, 2.1179529876187635)
+        mag_rtbe = estimate_magnitude(response, 3.61e4, 0.08, 2.197)
+        self.assertAlmostEqual(mag_rtbe, 1.1832677953138184)
+        mag_rnon = estimate_magnitude(response, 6.78e4, 0.125, 1.538)
+        self.assertAlmostEqual(mag_rnon, 1.4895395665022975)
+
     # XXX: Test for really big signal is missing, where the water level is
     # actually acting
     # def test_seisSimVsPitsa2(self):
@@ -352,6 +371,33 @@ class InvSimTestCase(unittest.TestCase):
             _h, f = evalresp(*args, **kwargs)
             self.assertEqual(len(f), nfft // 2 + 1)
 
+    def test_evalresp_specific_frequencies(self):
+        """
+        Test getting response for specific frequencies from evalresp
+        """
+        resp = os.path.join(self.path, 'RESP.CH._.HHZ.gz')
+        # test some frequencies (results taken from routine
+        # test_evalresp_bug_395)
+        freqs = [0.0, 0.0021303792075, 0.21303792075, 0.63911376225,
+                 2.1303792075, 21.303792075, 59.9978696208, 60.0]
+        expected = [0j, -38033660.9731 + 14722854.5862j,
+                    623756964.698 + 34705336.5587j,
+                    625815840.91 + 11748438.5949j,
+                    634173301.327 - 2261888.45356j,
+                    689435074.739 - 216615642.231j,
+                    -105.682658137 - 4360.67242023j,
+                    -101.693155157 - 4172.61059939j,
+                    ]
+        with NamedTemporaryFile() as fh:
+            tmpfile = fh.name
+            with gzip.open(resp) as f:
+                fh.write(f.read())
+            samprate = 120.0
+            t = UTCDateTime(2012, 9, 4, 5, 12, 15, 863300)
+            h = evalresp_for_frequencies(
+                1.0 / samprate, freqs, tmpfile, t, units='VEL')
+        np.testing.assert_allclose(h, expected)
+
     def test_evalresp_file_like_object(self):
         """
         Test evalresp with file like object
@@ -387,7 +433,8 @@ class InvSimTestCase(unittest.TestCase):
         filename = os.path.join(self.path, "segfaulting_RESPs",
                                 "RESP.IE.LLRI..EHZ")
         date = UTCDateTime(2003, 11, 1, 0, 0, 0)
-        with CatchOutput():
+        # raises C-level EVRESP ERROR
+        with SuppressOutput():
             self.assertRaises(ValueError, evalresp, t_samp=10.0, nfft=256,
                               filename=filename, date=date, station="LLRI",
                               channel="EHZ", network="IE", locid="*",
@@ -419,9 +466,9 @@ class InvSimTestCase(unittest.TestCase):
 
         # The RESP file only contains two channels.
         kwargs["channel"] = "BHZ"
-        with CatchOutput() as out:
+        # suppress a C-level "no response found" warning
+        with SuppressOutput():
             self.assertRaises(ValueError, evalresp, **kwargs)
-        self.assertIn(b"no response found for", out.stderr.lower())
 
     def test_evalresp_spline(self):
         """
