@@ -12,7 +12,6 @@ import math
 
 import numpy as np
 
-from obspy.core.util.decorator import deprecated
 from . import _DEFAULT_VALUES
 from .helper_classes import (CriticalDepth, DepthRange, SlownessLayer,
                              SlownessModelError, SplitLayerInfo, TimeDist)
@@ -115,16 +114,6 @@ class SlownessModel(object):
             desc += str(l) + "\n"
         return desc
 
-    @deprecated(
-        "'createSample' has been renamed to "  # noqa
-        "'create_sample'. Use that instead.")
-    def createSample(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'createSample' has been renamed to
-        'create_sample'. Use that instead.
-        '''
-        return self.create_sample(*args, **kwargs)
-
     def create_sample(self):
         """
         Create slowness-depth layers from a velocity model.
@@ -173,16 +162,6 @@ class SlownessModel(object):
         if self.debug:
             print("create_sample seems to be done successfully.")
 
-    @deprecated(
-        "'findCriticalPoints' has been renamed to "  # noqa
-        "'find_critical_points'. Use that instead.")
-    def findCriticalPoints(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'findCriticalPoints' has been renamed to
-        'find_critical_points'. Use that instead.
-        '''
-        return self.find_critical_points(*args, **kwargs)
-
     def find_critical_points(self):
         """
         Find all critical points within a velocity model.
@@ -206,8 +185,9 @@ class SlownessModel(object):
         min_s_so_far = 1.1e300
         # First remove any critical points previously stored
         # so these are effectively re-initialised... it's probably silly
-        self.critical_depths = np.empty(len(self.v_mod.layers),
-                                        dtype=CriticalDepth)
+        self.critical_depths = np.empty(
+            max(len(self.v_mod.layers), 3),
+            dtype=CriticalDepth)
         cd_count = 0
         self.high_slowness_layer_depths_p = []  # lists of DepthRange
         self.high_slowness_layer_depths_s = []
@@ -461,16 +441,6 @@ class SlownessModel(object):
 
         self.validate()
 
-    @deprecated(
-        "'getNumLayers' has been renamed to "  # noqa
-        "'get_num_layers'. Use that instead.")
-    def getNumLayers(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'getNumLayers' has been renamed to
-        'get_num_layers'. Use that instead.
-        '''
-        return self.get_num_layers(*args, **kwargs)
-
     def get_num_layers(self, is_p_wave):
         """
         Number of slowness layers.
@@ -488,26 +458,6 @@ class SlownessModel(object):
         else:
             return len(self.s_layers)
 
-    @deprecated(
-        "'findDepth_from_depths' has been renamed to "  # noqa
-        "'find_depth_from_depths'. Use that instead.")
-    def findDepth_from_depths(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'findDepth_from_depths' has been renamed to
-        'find_depth_from_depths'. Use that instead.
-        '''
-        return self.find_depth_from_depths(*args, **kwargs)
-
-    @deprecated(
-        "'findDepth_from_layers' has been renamed to "  # noqa
-        "'find_depth_from_layers'. Use that instead.")
-    def findDepth_from_layers(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'findDepth_from_layers' has been renamed to
-        'find_depth_from_layers'. Use that instead.
-        '''
-        return self.find_depth_from_layers(*args, **kwargs)
-
     def find_depth_from_depths(self, ray_param, top_depth, bot_depth,
                                is_p_wave):
         """
@@ -645,16 +595,6 @@ class SlownessModel(object):
             " top_critical_layer=" + str(top_critical_layer) +
             " bot_critical_layer=" + str(bot_critical_layer))
 
-    @deprecated(
-        "'toSlowness' has been renamed to "  # noqa
-        "'to_slowness'. Use that instead.")
-    def toSlowness(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'toSlowness' has been renamed to
-        'to_slowness'. Use that instead.
-        '''
-        return self.to_slowness(*args, **kwargs)
-
     def to_slowness(self, velocity, depth):
         """
         Convert velocity at some depth to slowness.
@@ -676,16 +616,6 @@ class SlownessModel(object):
                 "Maybe related to using S velocities in outer core?")
         return (self.radius_of_planet - depth) / velocity
 
-    @deprecated(
-        "'depthInFluid' has been renamed to "  # noqa
-        "'depth_in_fluid'. Use that instead.")
-    def depthInFluid(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'depthInFluid' has been renamed to
-        'depth_in_fluid'. Use that instead.
-        '''
-        return self.depth_in_fluid(*args, **kwargs)
-
     def interpolate(self, p, top_velocity, top_depth, slope):
         """
         Interpolate slowness to depth within a layer.
@@ -738,16 +668,6 @@ class SlownessModel(object):
             ret |= (elem.top_depth <= depth) & (depth < elem.bot_depth)
         return ret
 
-    @deprecated(
-        "'coarseSample' has been renamed to "  # noqa
-        "'coarse_sample'. Use that instead.")
-    def coarseSample(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'coarseSample' has been renamed to
-        'coarse_sample'. Use that instead.
-        '''
-        return self.coarse_sample(*args, **kwargs)
-
     def coarse_sample(self):
         """
         Create a coarse slowness sampling of the velocity model (v_mod).
@@ -879,16 +799,6 @@ class SlownessModel(object):
         for p in uniq:
             self.add_slowness(p, self.p_wave)
 
-    @deprecated(
-        "'layerNumberAbove' has been renamed to "  # noqa
-        "'layer_number_above'. Use that instead.")
-    def layerNumberAbove(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'layerNumberAbove' has been renamed to
-        'layer_number_above'. Use that instead.
-        '''
-        return self.layer_number_above(*args, **kwargs)
-
     def layer_number_above(self, depth, is_p_wave):
         """
         Find the index of the slowness layer that contains the given depth.
@@ -933,16 +843,6 @@ class SlownessModel(object):
 
         return found_layer_num
 
-    @deprecated(
-        "'layerNumberBelow' has been renamed to "  # noqa
-        "'layer_number_below'. Use that instead.")
-    def layerNumberBelow(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'layerNumberBelow' has been renamed to
-        'layer_number_below'. Use that instead.
-        '''
-        return self.layer_number_below(*args, **kwargs)
-
     def layer_number_below(self, depth, is_p_wave):
         """
         Find the index of the slowness layer that contains the given depth.
@@ -988,16 +888,6 @@ class SlownessModel(object):
 
         return found_layer_num
 
-    @deprecated(
-        "'getSlownessLayer' has been renamed to "  # noqa
-        "'get_slowness_layer'. Use that instead.")
-    def getSlownessLayer(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'getSlownessLayer' has been renamed to
-        'get_slowness_layer'. Use that instead.
-        '''
-        return self.get_slowness_layer(*args, **kwargs)
-
     def get_slowness_layer(self, layer, is_p_wave):
         """
         Return the Slowness_layer of the requested wave type.
@@ -1020,16 +910,6 @@ class SlownessModel(object):
         else:
             return self.s_layers[layer]
 
-    @deprecated(
-        "'addSlowness' has been renamed to "  # noqa
-        "'add_slowness'. Use that instead.")
-    def addSlowness(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'addSlowness' has been renamed to
-        'add_slowness'. Use that instead.
-        '''
-        return self.add_slowness(*args, **kwargs)
-
     def add_slowness(self, p, is_p_wave):
         """
         Add a ray parameter to the slowness sampling for the given wave type.
@@ -1112,16 +992,6 @@ class SlownessModel(object):
             self.s_layers = layers
             self.p_layers = other_layers
 
-    @deprecated(
-        "'ray_paramIncCheck' has been renamed to "  # noqa
-        "'ray_param_inc_check'. Use that instead.")
-    def ray_paramIncCheck(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'ray_paramIncCheck' has been renamed to
-        'ray_param_inc_check'. Use that instead.
-        '''
-        return self.ray_param_inc_check(*args, **kwargs)
-
     def ray_param_inc_check(self):
         """
         Check that no slowness layer's ray parameter interval is too large.
@@ -1153,16 +1023,6 @@ class SlownessModel(object):
                     self.add_slowness(newp, self.p_wave)
                     self.add_slowness(newp, self.s_wave)
 
-    @deprecated(
-        "'depthIncCheck' has been renamed to "  # noqa
-        "'depth_inc_check'. Use that instead.")
-    def depthIncCheck(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'depthIncCheck' has been renamed to
-        'depth_inc_check'. Use that instead.
-        '''
-        return self.depth_inc_check(*args, **kwargs)
-
     def depth_inc_check(self):
         """
         Check that no slowness layer is too thick.
@@ -1207,16 +1067,6 @@ class SlownessModel(object):
                     self.add_slowness(p, self.p_wave)
                     self.add_slowness(p, self.s_wave)
 
-    @deprecated(
-        "'distanceCheck' has been renamed to "  # noqa
-        "'distance_check'. Use that instead.")
-    def distanceCheck(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'distanceCheck' has been renamed to
-        'distance_check'. Use that instead.
-        '''
-        return self.distance_check(*args, **kwargs)
-
     def distance_check(self):
         """
         Check that no slowness layer is too wide or undersampled.
@@ -1350,16 +1200,6 @@ class SlownessModel(object):
                 print("Number of " + ("P" if curr_wave_type else "S") +
                       " slowness layers: " + str(j))
 
-    @deprecated(
-        "'depthInHighSlowness' has been renamed to "  # noqa
-        "'depth_in_high_slowness'. Use that instead.")
-    def depthInHighSlowness(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'depthInHighSlowness' has been renamed to
-        'depth_in_high_slowness'. Use that instead.
-        '''
-        return self.depth_in_high_slowness(*args, **kwargs)
-
     def depth_in_high_slowness(self, depth, ray_param, is_p_wave):
         """
         Determine if depth and slowness are within a high slowness zone.
@@ -1400,16 +1240,6 @@ class SlownessModel(object):
                     return True
         return False
 
-    @deprecated(
-        "'approxDistance' has been renamed to "  # noqa
-        "'approx_distance'. Use that instead.")
-    def approxDistance(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'approxDistance' has been renamed to
-        'approx_distance'. Use that instead.
-        '''
-        return self.approx_distance(*args, **kwargs)
-
     def approx_distance(self, slowness_turn_layer, p, is_p_wave):
         """
         Approximate distance for ray turning at the bottom of a layer.
@@ -1450,16 +1280,6 @@ class SlownessModel(object):
             td['dist'] = 2 * np.sum(dist)
         return td
 
-    @deprecated(
-        "'layerTimeDist' has been renamed to "  # noqa
-        "'layer_time_dist'. Use that instead.")
-    def layerTimeDist(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'layerTimeDist' has been renamed to
-        'layer_time_dist'. Use that instead.
-        '''
-        return self.layer_time_dist(*args, **kwargs)
-
     def layer_time_dist(self, spherical_ray_param, layer_num, is_p_wave,
                         check=True, allow_turn=False):
         """
@@ -1702,16 +1522,6 @@ class SlownessModel(object):
 
         return time, dist
 
-    @deprecated(
-        "'fixCriticalPoints' has been renamed to "  # noqa
-        "'fix_critical_points'. Use that instead.")
-    def fixCriticalPoints(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'fixCriticalPoints' has been renamed to
-        'fix_critical_points'. Use that instead.
-        '''
-        return self.fix_critical_points(*args, **kwargs)
-
     def fix_critical_points(self):
         """
         Reset the slowness layers that correspond to critical points.
@@ -1824,16 +1634,6 @@ class SlownessModel(object):
         # Everything seems OK.
         return True
 
-    @deprecated(
-        "'getMinTurnRayParam' has been renamed to "  # noqa
-        "'get_min_turn_ray_param'. Use that instead.")
-    def getMinTurnRayParam(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'getMinTurnRayParam' has been renamed to
-        'get_min_turn_ray_param'. Use that instead.
-        '''
-        return self.get_min_turn_ray_param(*args, **kwargs)
-
     def get_min_turn_ray_param(self, depth, is_p_wave):
         """
         Find minimum slowness, turning but not reflected, at or above a depth.
@@ -1874,16 +1674,6 @@ class SlownessModel(object):
                                                   self.radius_of_planet)
         return min_p_so_far
 
-    @deprecated(
-        "'getMinRayParam' has been renamed to "  # noqa
-        "'get_min_ray_param'. Use that instead.")
-    def getMinRayParam(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'getMinRayParam' has been renamed to
-        'get_min_ray_param'. Use that instead.
-        '''
-        return self.get_min_ray_param(*args, **kwargs)
-
     def get_min_ray_param(self, depth, is_p_wave):
         """
         Find minimum slowness, turning or reflected, at or above a depth.
@@ -1912,16 +1702,6 @@ class SlownessModel(object):
                                s_layer_below['top_p'])
         return min_p_so_far
 
-    @deprecated(
-        "'splitLayer' has been renamed to "  # noqa
-        "'split_layer'. Use that instead.")
-    def splitLayer(self, *args, **kwargs):
-        '''
-        DEPRECATED: 'splitLayer' has been renamed to
-        'split_layer'. Use that instead.
-        '''
-        return self.split_layer(*args, **kwargs)
-
     def split_layer(self, depth, is_p_wave):
         """
         Split a slowness layer into two slowness layers.
